##--------------------------------------------------------------------------------------------------------
## SCRIPT : Null Model
##
## Authors : Marine Busson, Matthieu Authier, Christophe Barbraud, Paul Tixier, Ryan R. Reisinger, Anas Janc, Christophe Guinet
## Last update : 2019-05-02
##
## R version 3.5.2 (2018-12-20) -- "Eggshell Igloo"
## Copyright (C) 2018 The R Foundation for Statistical Computing
## Platform: x86_64-w64-mingw32/x64 (64-bit)
##--------------------------------------------------------------------------------------------------------

lapply(c("ggplot2", "ggthemes", "dplyr", "reshape2", "mvtnorm", "coda", "rjags"), 
       library, character.only = TRUE)

rm(list = ls())
## useful functions with jags
param <- function(jagsfit){ do.call("rbind", lapply(jagsfit, as.data.frame)) }

gbr <- function(jagsfit, theta = NULL) {
  posterior <- param(jagsfit)
  if(!is.null(theta)) {
    posterior <- posterior[, theta]
  }
  n <- nrow(posterior)/length(jagsfit)
  return(coda::gelman.diag(coda::as.mcmc.list(lapply(c(1:length(jagsfit)), function(x) { return(coda::as.mcmc(posterior[n*(x-1)+c(1:n), ])) } ))))
}

get_summary <- function(x) {
  x <- coda::as.mcmc(x)
  return(c(mean(x), sd(x), median(x), coda::HPDinterval(x, prob = 0.95)))
}

WorkDir <- getwd() # insert path to working directory
DataDir <- paste(WorkDir, "data", sep = "/")
OutDir <- paste(WorkDir, "output", sep = "/")

set.seed(20190218)
load(paste(DataDir, "CrozetOrca.RData", sep = "/"))
orca <- as.data.frame(orca)
n_ind <- nrow(orca)
n_obs <- length(1987:2014)

ALIVE <- OBS <- matrix(0, nrow = n_ind, ncol = n_obs)
for(j in 1:n_obs) {
  if(any(names(orca) == as.character(1987:2014)[j])) {
    OBS[, j] <- orca[, as.character(1987:2014)[j]]
  }
}; rm(j)

for(i in 1:n_ind) {
  ALIVE[i, min(which(OBS[i, ] == 1)):n_obs] <- NA
}; rm(i)

### Augmented data
jagsdata <- list(n_ind = n_ind,
                 n_year = n_obs,
                 OBS = OBS,
                 FIRST = as.numeric(sapply(1:n_ind, function(id) {min(which(OBS[id, ] == 1))} )),
                 CUMUL = 1 + c(0, cumsum(as.numeric(sapply(1:n_ind, function(id) {n_obs - min(which(OBS[id, ] == 1)) + 1} )))[-n_ind]),
                 ALIVE = ALIVE,
                 FIELDWORK = c(1, 1, 1, 1, 0, 0, 1, 1, 0, rep(1, 19))
                 )

### Call Jags from R
jagsify <- function(jagsdata, ni = 2000, nb = 1000, nc = 3, nt = 10) {
  ### function to generate initial values
  initjags <- function(datalist) {
    FIRST <- LAST <- rep(NA, datalist$n_ind)
    ALIVE <- matrix(1, nrow = datalist$n_ind, ncol = datalist$n_year)
    
    for(i in 1:datalist$n_ind) {
      FIRST[i] <- min(which(datalist$OBS[i, ] == 1))
      LAST[i] <- max(which(datalist$OBS[i, ] == 1))
    }
    for(i in 1:datalist$n_ind) {
      if(FIRST[i] != 1) {
        ALIVE[i, 1:(FIRST[i] - 1)] <- NA
      }
    }
    list(sigma_year = 0.1 * abs(rnorm(2)),
         etap = cbind(rnorm(datalist$n_year),rnorm(datalist$n_year)),
         beta = 1.5 * rnorm(1),
         gamma = c(1.5, 1.0) * rnorm(2),
         ALIVE = ALIVE
         )
  }
  
  writeLines(paste("\tCapture-Recapture model", sep = " "))
  if(is.null(nb)) { nb <- floor(ni / 2) }
  ini <- lapply(1:nc, function(id) { initjags(datalist = jagsdata) })
  Time_simul <- numeric(2)
  Start <- Sys.time()
  
  warmup <- jags.model(paste(WorkDir, "data", "M0.bug", sep = "/"),
                       data = jagsdata,
                       n.chains = nc,
                       n.adapt = nb, 
                       inits = ini, 
                       quiet = FALSE
                       )
  
  Time_simul[1] <- difftime(Sys.time(), Start, units = "sec") # Time difference
  
  Start <- Sys.time()
  model_out <- coda.samples(model = warmup,
                            variable.names = c("sigma_year", "etap", "gamma", "beta", "log_lik"),
                            n.iter = (ni - nb) * nt,
                            thin = nt
                            )
  
  Time_simul[2] <- difftime(Sys.time(), Start, units = "sec") # Time difference

  return(list(time_elapsed = Time_simul,
              # initial_values = ini,
              jagsmodel = model_out
              )
         )
}

null <- jagsify(jagsdata = jagsdata)

save.image(paste(OutDir, "mod0.RData", sep = "/"), safe = TRUE)

### extract posterior
theta <- param(jagsfit = null$jagsmodel)
### check convergence quickly
conv <- gbr(null$jagsmodel, 
      theta = c("beta", "gamma[1]", "gamma[2]", "sigma_year[1]", "sigma_year[2]",
                "etap[1,1]", "etap[2,1]", "etap[3,1]", "etap[4,1]", "etap[5,1]", "etap[6,1]", "etap[7,1]",
                "etap[8,1]", "etap[9,1]", "etap[10,1]", "etap[11,1]", "etap[12,1]", "etap[13,1]", 
                "etap[14,1]", "etap[15,1]", "etap[16,1]", "etap[17,1]", "etap[18,1]", "etap[19,1]", 
                "etap[20,1]", "etap[21,1]", "etap[22,1]", "etap[23,1]", "etap[24,1]", "etap[25,1]",
                "etap[26,1]", "etap[27,1]", "etap[28,1]",
                "etap[1,2]", "etap[2,2]", "etap[3,2]", "etap[4,2]", "etap[5,2]", "etap[6,2]", "etap[7,2]",
                "etap[8,2]", "etap[9,2]", "etap[10,2]", "etap[11,2]", "etap[12,2]", "etap[13,2]", 
                "etap[14,2]", "etap[15,2]", "etap[16,2]", "etap[17,2]", "etap[18,2]", "etap[19,2]", 
                "etap[20,2]", "etap[21,2]", "etap[22,2]", "etap[23,2]", "etap[24,2]", "etap[25,2]",
                "etap[26,2]", "etap[27,2]", "etap[28,2]")
      )
conv

### waic
loo::waic(as.matrix(theta[, grep('log_lik', names(theta))]))
