##--------------------------------------------------------------------------------------------------------
## SCRIPT : All other models with Jags
##
## Authors : Marine Bussona, Matthieu Authier, Christophe Barbraud, Paul Tixier, Ryan R. Reisinger, Anas Janc, Christophe Guinet
## Last update : 2019-05-02
##
## R version 3.5.2 (2018-12-20) -- "Eggshell Igloo"
## Copyright (C) 2018 The R Foundation for Statistical Computing
## Platform: x86_64-w64-mingw32/x64 (64-bit)
##--------------------------------------------------------------------------------------------------------

lapply(c("ggplot2", "ggthemes", "dplyr", "reshape2", "mvtnorm", "coda", "rjags"), 
       library, character.only = TRUE)

rm(list = ls())
## useful functions with jags
param <- function(jagsfit){ do.call("rbind", lapply(jagsfit, as.data.frame)) }

gbr <- function(jagsfit, theta = NULL) {
  posterior <- param(jagsfit)
  if(!is.null(theta)) {
    posterior <- posterior[, theta]
  }
  n <- nrow(posterior)/length(jagsfit)
  return(coda::gelman.diag(coda::as.mcmc.list(lapply(c(1:length(jagsfit)), function(x) { return(coda::as.mcmc(posterior[n*(x-1)+c(1:n), ])) } ))))
}

get_summary <- function(x) {
  x <- coda::as.mcmc(x)
  return(c(mean(x), sd(x), median(x), coda::HPDinterval(x, prob = 0.95)))
}

WorkDir <- getwd() # insert path to working directory
DataDir <- paste(WorkDir, "data", sep = "/")
OutDir <- paste(WorkDir, "output", sep = "/")

set.seed(20190218)
load(paste(DataDir, "CrozetOrca.RData", sep = "/"))
orca <- as.data.frame(orca)
n_ind <- nrow(orca)
n_obs <- length(1987:2014)

ALIVE <- OBS <- matrix(0, nrow = n_ind, ncol = n_obs)
for(j in 1:n_obs) {
  if(any(names(orca) == as.character(1987:2014)[j])) {
    OBS[, j] <- orca[, as.character(1987:2014)[j]]
  }
}; rm(j)

for(i in 1:n_ind) {
  ALIVE[i, min(which(OBS[i, ] == 1)):n_obs] <- NA
}; rm(i)

photo <- c(1460, 956, 864, 1046, NA, NA, 1, 1, NA, 1, 23, 1046, 105, 2591, 1, 46, 678916, 764, 44721, 14373, 82726, 26061, 197249, 85001, 71588, 88733, 39623, 21578)
std_photo <- (photo - mean(photo, na.rm = TRUE)) / sd(photo, na.rm = TRUE)
### Augmented data
jagsdata <- list(n_ind = n_ind,
                 n_year = n_obs,
                 OBS = OBS,
                 EFFORT = ifelse(is.na(std_photo), 0, std_photo),
                 SIZE = as.numeric(scale(orca$SIZE)),
                 HWI = as.numeric(scale(orca$HWI)),
                 n_group = length(unique(orca$IDgrp)),
                 GROUP = orca$IDgrp,
                 FIRST = as.numeric(sapply(1:n_ind, function(id) {min(which(OBS[id, ] == 1))} )),
                 CUMUL = 1 + c(0, cumsum(as.numeric(sapply(1:n_ind, function(id) {n_obs - min(which(OBS[id, ] == 1)) + 1} )))[-n_ind]),
                 ALIVE = ALIVE,
                 FIELDWORK = c(1, 1, 1, 1, 0, 0, 1, 1, 0, rep(1, 19))
                 )

### Call Jags from R
jagsify <- function(jagsdata, ni = 2000, nb = 1000, nc = 3, nt = 10) {
  ### function to generate initial values
  initjags <- function(datalist) {
    FIRST <- LAST <- rep(NA, datalist$n_ind)
    ALIVE <- matrix(1, nrow = datalist$n_ind, ncol = datalist$n_year)
    
    for(i in 1:datalist$n_ind) {
      FIRST[i] <- min(which(datalist$OBS[i, ] == 1))
      LAST[i] <- max(which(datalist$OBS[i, ] == 1))
    }
    for(i in 1:datalist$n_ind) {
      if(FIRST[i] != 1) {
        ALIVE[i, 1:(FIRST[i] - 1)] <- NA
      }
    }
    list(sigma_year = 0.1 * abs(rnorm(1)),
         sigma_group = 0.1 * abs(rnorm(1)),
         etap = rnorm(datalist$n_year),
         etag = rnorm(datalist$n_group),
         beta = c(1.5, 1.0, 1.0) * rnorm(3),
         gamma = c(1.5, 1.0, 1.0) * rnorm(3),
         ALIVE = ALIVE
         )
  }
  
  writeLines(paste("\tCapture-Recapture model", sep = " "))
  if(is.null(nb)) { nb <- floor(ni / 2) }
  ini <- lapply(1:nc, function(id) { initjags(datalist = jagsdata) })
  Time_simul <- numeric(2)
  Start <- Sys.time()
  
  warmup <- jags.model(paste(WorkDir, "data", "M.bug", sep = "/"),
                       data = jagsdata,
                       n.chains = nc,
                       n.adapt = nb, 
                       inits = ini, 
                       quiet = FALSE
                       )
  
  Time_simul[1] <- difftime(Sys.time(), Start, units = "sec") # Time difference
  
  Start <- Sys.time()
  model_out <- coda.samples(model = warmup,
                            variable.names = c("sigma_year", "sigma_group", "etag", "etap", "gamma", "beta", "log_lik"),
                            n.iter = (ni - nb) * nt,
                            thin = nt
                            )
  
  Time_simul[2] <- difftime(Sys.time(), Start, units = "sec") # Time difference

  return(list(time_elapsed = Time_simul,
              # initial_values = ini,
              jagsmodel = model_out
              )
         )
}

### fit model
w <- matrix(c(0, 0, 0, 
              0, 0, 0, 
              1, 0, 0,
              0, 1, 0,
              0, 0, 1,
              1, 0, 1,
              0, 1, 1,
              0, 0, 0,
              1, 0, 0,
              1, 1, 0), nrow = 10, ncol = 3, byrow = TRUE)
z <- matrix(c(1, 0, 0, 
              0, 1, 0, 
              1, 0, 0, 
              1, 0, 0,
              1, 0, 0,
              1, 0, 0,
              1, 0, 0,
              1, 0, 1,
              1, 0, 1,
              1, 0, 1), nrow = 10, ncol = 3, byrow = TRUE)

results <- vector(length = 10, mode = 'list')
for(i in 1:nrow(w)) {
  temp <- jagsdata
  temp$w <- w[i, ]
  temp$z <- z[i, ]
  results[[i]] <- jagsify(jagsdata = temp)
  rm(temp)
  save.image(paste(OutDir, "mods.RData", sep = "/"), safe = TRUE)
}; rm(i)

save.image(paste(OutDir, "mods.RData", sep = "/"), safe = TRUE)

### check convergence
conv <- function(l) {
  gbr(l$jagsmodel, 
      theta = c("beta[1]", "beta[2]", "gamma[1]", "gamma[2]", "sigma_year", "sigma_group",
                "etap[1]", "etap[2]", "etap[3]", "etap[4]", "etap[5]", "etap[6]", "etap[7]",
                "etap[8]", "etap[9]", "etap[10]", "etap[11]", "etap[12]", "etap[13]", 
                "etap[14]", "etap[15]", "etap[16]", "etap[17]", "etap[18]", "etap[19]", 
                "etap[20]", "etap[21]", "etap[22]", "etap[23]", "etap[24]", "etap[25]",
                "etap[26]", "etap[27]", "etap[28]")
      )
}

allconv <- lapply(results, conv)

### potential scale reduction factor
do.call('cbind', lapply(allconv, function(l) { l$psrf[, 2] }))

### multivariate potential scale reduction factor
do.call('c', lapply(allconv, function(l) { l$mpsrf }))

### extract posterior
waic_jags <- function(id) {
  theta <- param(jagsfit = results[[id]]$jagsmodel)
  return(loo::waic(as.matrix(theta[, grep('log_lik', names(theta))])))
}

compare_jags <- function(m1, m2) {
  theta1 <- param(jagsfit = results[[m1]]$jagsmodel)
  theta2 <- param(jagsfit = results[[m2]]$jagsmodel)
  loo::compare(loo::waic(as.matrix(theta1[, grep('log_lik', names(theta1))])),
               loo::waic(as.matrix(theta2[, grep('log_lik', names(theta2))]))
               )
}

compare_jags(1, 2) # 1 is better
compare_jags(1, 3) # 3 is better
compare_jags(3, 4) # 3 is better
compare_jags(3, 5) # 3 is better
compare_jags(3, 6) # 6 is better
compare_jags(6, 7) # 6 is better
compare_jags(6, 8) # 8 is better
compare_jags(8, 9) # 9 is better
compare_jags(9, 10)# 10 is better

### model 10 is the best
### look at effect of HWI on survival
theta <- param(jagsfit = results[[10]]$jagsmodel)
std_hwi <- (seq(0, 0.06, length.out = 1000) - mean(orca$HWI))/sd(orca$HWI)
std_group <- (seq(1, 9, length.out = 1000) - mean(orca$SIZE))/sd(orca$SIZE)

p_hwi <- plogis(as.matrix(theta[, c('beta[1]', 'beta[2]')]) %*% 
                  rbind(rep(1, 1000), std_hwi)
                )
p_group <- plogis(as.matrix(theta[, c('beta[1]', 'beta[3]')]) %*% 
                    rbind(rep(1, 1000), std_group)
                  )

post <- data.frame(x = c(seq(0, 0.06, length.out = 1000), seq(1, 9, length.out = 1000)),
                   y = c(apply(p_hwi, 2, mean), apply(p_group, 2, mean)),
                   lower = c(apply(p_hwi, 2, quantile, probs = 0.025), 
                             apply(p_group, 2, quantile, probs = 0.025)
                             ),
                   upper = c(apply(p_hwi, 2, quantile, probs = 0.975), 
                             apply(p_group, 2, quantile, probs = 0.975)
                             ),
                   effect = rep(c("HWI", "Group Size"), each = 1000)
                   )

library(tidyverse)

post %>% 
  ggplot(aes(x = x, y = y)) +
  geom_ribbon(aes(x = x, ymin = lower, ymax = upper), alpha = 0.3, fill = "steelblue") +
  geom_line(color = "midnightblue") +
  facet_wrap(~effect, ncol = 2, scales = "free_x") +
  ylab(bquote(phi)) + xlab("covariate value") +
  theme_bw()

### detection probabilities
pD <- plogis(matrix(rep(theta[, 'gamma[1]'], each = 28), byrow = TRUE, ncol = 28) + 
  as.matrix(theta[, grep('etap', names(theta))])) %*% diag(x = jagsdata$FIELDWORK, ncol = 28)
baseline <- round(t(apply(pD, 2, get_summary))[, c(1, 4:5)], 3)

pD <- plogis(matrix(rep(theta[, 'gamma[1]'] + theta[, 'gamma[2]'], each = 28), byrow = TRUE, ncol = 28) + 
               as.matrix(theta[, grep('etap', names(theta))])) %*% diag(x = jagsdata$FIELDWORK, ncol = 28)
trap_happy <- round(t(apply(pD, 2, get_summary))[, c(1, 4:5)], 3)
